import { Body, Controller, Get, Ip, Post } from '@nestjs/common';
import {
  ApiOperation,
  ApiProperty,
  ApiResponse,
  ApiTags,
} from '@nestjs/swagger';
import { Auth, AuthData, NoAuthentication } from './authdata.decorator';
import { AuthService } from './auth.service';
import { AuthInfo } from 'src/app.controller';
import { AuthenticationData } from './auth.guard';
import { DatabaseService } from 'src/database/database.service';

export class LoginData {
  @ApiProperty({ description: 'The user ID you are trying to log in as' })
  user: string;
}

export class LoginResponse {
  @ApiProperty()
  token: string;
  @ApiProperty()
  otp: string;
}

@Controller('auth')
@ApiTags('authentication')
export class AuthController {
  constructor(private auth: AuthService, private db: DatabaseService) {}

  @Get('/')
  @Auth()
  @ApiOperation({ summary: 'Check authentication status' })
  @ApiResponse({ type: AuthInfo, status: 200 })
  async getAuth(@AuthData() auth: AuthenticationData): Promise<AuthInfo> {
    return {
      // Since the AuthGuard stops the request otherwise, we can assume the user is authenticated
      authenticated: true,
      user: auth.user,
      god: auth.god || false,
      profile: auth.user
        ? (await this.db.getMessengers().getuserInfo({ id: auth.user })) || null
        : null,
    };
  }

  @Post('login')
  @NoAuthentication()
  @ApiResponse({ type: LoginResponse })
  async login(
    @Body() data: LoginData,
    @Ip() ip: string,
  ): Promise<LoginResponse> {
    return await this.auth.createLoginRequest(data.user, ip);
  }
}
