import localforage from "localforage";
import { API } from "oapilib";

type CredentialsData = {
    token: string | null;
};

type PendingAuthData = {
    token: string;
    otp: string;
};

export const apiWithCredentials = (token?: string) =>
    token
        ? new API({
              authentication: { headers: { Authorization: `Bearer ${token}` } },
          })
        : new API();

export async function setCredentials({ token }: CredentialsData) {
    await localforage.setItem("api_token", token);
}

export async function getCredentials(): Promise<CredentialsData> {
    return {
        token: await localforage.getItem("api_token"),
    };
}

export async function setPendingAuthData(data: PendingAuthData | null) {
    await Promise.all([
        localforage.setItem("auth_pending_otp", data?.otp),
        localforage.setItem("auth_pending_token", data?.token),
    ]);
}

export async function getPendingAuthData(): Promise<PendingAuthData | null> {
    const [otp, token] = await Promise.all([
        localforage.getItem<string>("auth_pending_otp"),
        localforage.getItem<string>("auth_pending_token"),
    ]);

    if (otp && token) return { otp, token };

    return null;
}
