"use client";

import { useRouter } from "next/navigation";
import { useEffect, useState } from "react";
import { apiWithCredentials, getCredentials } from "./auth";
import { useBearStore } from "./useState";

export default function useAuth({ redirect = true } = {}) {
    const state = useBearStore();
    const router = useRouter();

    const [auth, setAuth] = useState(state.auth);

    useEffect(() => {
        (async () => {
            const creds = await getCredentials();

            if (!creds.token && auth) {
                setAuth(undefined);
            }

            if (!auth) {
                if (creds.token) {
                    state.api = apiWithCredentials(creds.token);

                    try {
                        const auth = await state.api.get("/auth");
                        state.auth = auth;
                        setAuth(auth);
                        return;
                    } catch (e) {
                        console.log(
                            e,
                            " - Treating this as authentication error.",
                        );
                    }
                }

                if (redirect) router.push("/auth");
            }
        })();
    });

    return auth;
}
